﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Scheduling.MRCPSP {

  [Item("MRCPSP Improver", "Improves a solution by calling all MRCPSP improvement operators.")]
  [StorableClass]
  public sealed class MrcpspImprover : SingleSuccessorOperator, IImprovementOperator {

    public MrcpspImprover() {
      Parameters.Add(new LookupParameter<ItemCollection<IItem>>("Operators", ""));
    }

    [StorableConstructor]
    private MrcpspImprover(bool deserializing) : base(deserializing) { }

    private MrcpspImprover(MrcpspImprover original, Cloner cloner) : base(original, cloner) {
    }

    public ILookupParameter<ItemCollection<IItem>> OperatorsParameter =>
      (ILookupParameter<ItemCollection<IItem>>)Parameters["Operators"];

    public override IOperation Apply() {
      var operators = OperatorsParameter.ActualValue;
      var operations = operators
        .OfType<IImprovementOperator>()
        .Where(o => !(o is MrcpspImprover))
        .Select(op => ExecutionContext.CreateOperation(op));
      return new OperationCollection(operations);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MrcpspImprover(this, cloner);
    }
  }
}
